@@ -1,4 +1,5 @@
 import json
+import time
 from re import match
 from typing import Any, Dict, List, Optional, Union
 
@@ -27,6 +28,7 @@ class ApiGatewayRestApi(Resource):
         "MinimumCompressionSize": GeneratedProperty(),
         "Mode": GeneratedProperty(),
         "ApiKeySourceType": GeneratedProperty(),
+        "Tags": GeneratedProperty(),
     }
 
     Body: Optional[Dict[str, Any]]
@@ -41,6 +43,7 @@ class ApiGatewayRestApi(Resource):
     MinimumCompressionSize: Optional[PassThrough]
     Mode: Optional[PassThrough]
     ApiKeySourceType: Optional[PassThrough]
+    Tags: Optional[PassThrough]
 
     runtime_attrs = {"rest_api_id": lambda self: ref(self.logical_id)}
 
@@ -89,16 +92,17 @@ class ApiGatewayDeployment(Resource):
 
     runtime_attrs = {"deployment_id": lambda self: ref(self.logical_id)}
 
-    def make_auto_deployable(
+    def make_auto_deployable(  # noqa: too-many-arguments
         self,
         stage: ApiGatewayStage,
         openapi_version: Optional[Union[Dict[str, Any], str]] = None,
         swagger: Optional[Dict[str, Any]] = None,
         domain: Optional[Dict[str, Any]] = None,
         redeploy_restapi_parameters: Optional[Any] = None,
+        always_deploy: Optional[bool] = False,
     ) -> None:
         """
-        Sets up the resource such that it will trigger a re-deployment when Swagger changes
+        Sets up the resource such that it will trigger a re-deployment when Swagger changes or always_deploy is true
         or the openapi version changes or a domain resource changes.
 
         :param stage: The ApiGatewayStage object which will be re-deployed
@@ -126,11 +130,15 @@ class ApiGatewayDeployment(Resource):
         # The keyword "Deployment" is removed and all the function names associated with api is obtained
         if function_names and function_names.get(self.logical_id[:-10], None):
             hash_input.append(function_names.get(self.logical_id[:-10], ""))
+        if always_deploy:
+            # We just care that the hash changes every time
+            # Using int so tests are a little more robust; don't think the Python spec defines default precision
+            hash_input = [str(int(time.time()))]
         data = self._X_HASH_DELIMITER.join(hash_input)
         generator = logical_id_generator.LogicalIdGenerator(self.logical_id, data)
         self.logical_id = generator.gen()
         digest = generator.get_hash(length=40)
-        self.Description = "RestApi deployment id: {}".format(digest)
+        self.Description = f"RestApi deployment id: {digest}"
         stage.update_deployment_ref(self.logical_id)
 
 
@@ -149,7 +157,7 @@ class ApiGatewayResponse:
             for response_parameter_key in response_parameters:
                 if response_parameter_key not in ApiGatewayResponse.ResponseParameterProperties:
                     raise InvalidResourceException(
-                        api_logical_id, "Invalid gateway response parameter '{}'".format(response_parameter_key)
+                        api_logical_id, f"Invalid gateway response parameter '{response_parameter_key}'"
                     )
 
         status_code_str = self._status_code_string(status_code)  # type: ignore[no-untyped-call]
@@ -208,6 +216,7 @@ class ApiGatewayDomainName(Resource):
         "MutualTlsAuthentication": GeneratedProperty(),
         "SecurityPolicy": GeneratedProperty(),
         "CertificateArn": GeneratedProperty(),
+        "Tags": GeneratedProperty(),
         "OwnershipVerificationCertificateArn": GeneratedProperty(),
     }
 
@@ -217,6 +226,7 @@ class ApiGatewayDomainName(Resource):
     MutualTlsAuthentication: Optional[Dict[str, Any]]
     SecurityPolicy: Optional[PassThrough]
     CertificateArn: Optional[PassThrough]
+    Tags: Optional[PassThrough]
     OwnershipVerificationCertificateArn: Optional[PassThrough]
 
 
@@ -260,6 +270,7 @@ class ApiGatewayApiKey(Resource):
         "Enabled": GeneratedProperty(),
         "GenerateDistinctId": GeneratedProperty(),
         "Name": GeneratedProperty(),
+        "Tags": GeneratedProperty(),
         "StageKeys": GeneratedProperty(),
         "Value": GeneratedProperty(),
     }
