@@ -5,14 +5,16 @@ from typing import Any, Dict, List, Tuple
 from samtranslator.metrics.method_decorator import cw_timer
 from samtranslator.model.exceptions import InvalidEventException, InvalidResourceException
 from samtranslator.model.iam import IAMRole, IAMRolePolicies
+from samtranslator.model.intrinsics import fnJoin, is_intrinsic
 from samtranslator.model.resource_policies import ResourcePolicies
 from samtranslator.model.role_utils import construct_role_for_resource
 from samtranslator.model.s3_utils.uri_parser import parse_s3_uri
-from samtranslator.model.stepfunctions import StepFunctionsStateMachine
-from samtranslator.model.intrinsics import fnJoin
+from samtranslator.model.stepfunctions.resources import (
+    StepFunctionsStateMachine,
+    StepFunctionsStateMachineAlias,
+    StepFunctionsStateMachineVersion,
+)
 from samtranslator.model.tags.resource_tagging import get_tag_list
-
-from samtranslator.model.intrinsics import is_intrinsic
 from samtranslator.model.xray_utils import get_xray_managed_policy_name
 from samtranslator.utils.cfn_dynamic_references import is_dynamic_reference
 
@@ -26,7 +28,7 @@ class StateMachineGenerator:
         "Specify either 'Role' or 'Policies' (but not both at the same time) or neither of them"
     )
 
-    def __init__(  # type: ignore[no-untyped-def]
+    def __init__(  # type: ignore[no-untyped-def] # noqa: too-many-arguments
         self,
         logical_id,
         depends_on,
@@ -49,6 +51,9 @@ class StateMachineGenerator:
         tags=None,
         resource_attributes=None,
         passthrough_resource_attributes=None,
+        get_managed_policy_map=None,
+        auto_publish_alias=None,
+        deployment_preference=None,
     ):
         """
         Constructs an State Machine Generator class that generates a State Machine resource
@@ -74,6 +79,8 @@ class StateMachineGenerator:
         :param tags: Tags to be associated with the State Machine resource
         :param resource_attributes: Resource attributes to add to the State Machine resource
         :param passthrough_resource_attributes: Attributes such as `Condition` that are added to derived resources
+        :param auto_publish_alias: Name of the state machine alias to automatically create and update
+        :deployment_preference: Settings to enable gradual state machine deployments
         """
         self.logical_id = logical_id
         self.depends_on = depends_on
@@ -100,6 +107,9 @@ class StateMachineGenerator:
             logical_id, depends_on=depends_on, attributes=resource_attributes
         )
         self.substitution_counter = 1
+        self.get_managed_policy_map = get_managed_policy_map
+        self.auto_publish_alias = auto_publish_alias
+        self.deployment_preference = deployment_preference
 
     @cw_timer(prefix="Generator", name="StateMachine")
     def to_cloudformation(self):  # type: ignore[no-untyped-def]
@@ -140,8 +150,6 @@ class StateMachineGenerator:
         if self.role:
             self.state_machine.RoleArn = self.role
         else:
-            if self.policies and not self.managed_policy_map:
-                raise Exception("Managed policy map is empty, but should not be.")
             if not self.policies:
                 self.policies = []
             execution_role = self._construct_role()
@@ -154,6 +162,9 @@ class StateMachineGenerator:
         self.state_machine.TracingConfiguration = self.tracing
         self.state_machine.Tags = self._construct_tag_list()
 
+        managed_traffic_shifting_resources = self._generate_managed_traffic_shifting_resources()
+        resources.extend(managed_traffic_shifting_resources)
+
         event_resources = self._generate_event_resources()
         resources.extend(event_resources)
 
@@ -202,8 +213,7 @@ class StateMachineGenerator:
         # Indenting and then splitting the JSON-encoded string for readability of the state machine definition in the CloudFormation translated resource.
         # Separators are passed explicitly to maintain trailing whitespace consistency across Py2 and Py3
         definition_lines = json.dumps(definition_dict, sort_keys=True, indent=4, separators=(",", ": ")).split("\n")
-        definition_string = fnJoin("\n", definition_lines)
-        return definition_string
+        return fnJoin("\n", definition_lines)
 
     def _construct_role(self) -> IAMRole:
         """
@@ -222,7 +232,7 @@ class StateMachineGenerator:
             policy_template_processor=None,
         )
 
-        execution_role = construct_role_for_resource(
+        return construct_role_for_resource(
             resource_logical_id=self.logical_id,
             role_path=self.role_path,
             attributes=self.passthrough_resource_attributes,
@@ -231,8 +241,8 @@ class StateMachineGenerator:
             resource_policies=state_machine_policies,
             tags=self._construct_tag_list(),
             permissions_boundary=self.permissions_boundary,
+            get_managed_policy_map=self.get_managed_policy_map,
         )
-        return execution_role
 
     def _construct_tag_list(self) -> List[Dict[str, Any]]:
         """
@@ -244,6 +254,72 @@ class StateMachineGenerator:
         sam_tag = {self._SAM_KEY: self._SAM_VALUE}
         return get_tag_list(sam_tag) + get_tag_list(self.tags)
 
+    def _construct_version(self) -> StepFunctionsStateMachineVersion:
+        """Constructs a state machine version resource that will be auto-published when the revision id of the state machine changes.
+
+        :return: Step Functions state machine version resource
+        """
+
+        # Unlike Lambda function versions, state machine versions do not need a hash suffix because
+        # they are always replaced when their corresponding state machine is updated.
+        # I.e. A SAM StateMachine resource will never have multiple version resources at the same time.
+        logical_id = f"{self.logical_id}Version"
+        attributes = self.passthrough_resource_attributes.copy()
+
+        # Both UpdateReplacePolicy and DeletionPolicy are needed to protect previous version from deletion
+        # to ensure gradual deployment works.
+        if "DeletionPolicy" not in attributes:
+            attributes["DeletionPolicy"] = "Retain"
+        if "UpdateReplacePolicy" not in attributes:
+            attributes["UpdateReplacePolicy"] = "Retain"
+
+        state_machine_version = StepFunctionsStateMachineVersion(logical_id=logical_id, attributes=attributes)
+        state_machine_version.StateMachineArn = self.state_machine.get_runtime_attr("arn")
+        state_machine_version.StateMachineRevisionId = self.state_machine.get_runtime_attr("state_machine_revision_id")
+
+        return state_machine_version
+
+    def _construct_alias(self, version: StepFunctionsStateMachineVersion) -> StepFunctionsStateMachineAlias:
+        """Constructs a state machine alias resource pointing to the given state machine version.
+        :return: Step Functions state machine alias resource
+        """
+        logical_id = f"{self.logical_id}Alias{self.auto_publish_alias}"
+        attributes = self.passthrough_resource_attributes
+
+        state_machine_alias = StepFunctionsStateMachineAlias(logical_id=logical_id, attributes=attributes)
+        state_machine_alias.Name = self.auto_publish_alias
+
+        state_machine_version_arn = version.get_runtime_attr("arn")
+
+        deployment_preference = {}
+        if self.deployment_preference:
+            deployment_preference = self.deployment_preference
+        else:
+            deployment_preference["Type"] = "ALL_AT_ONCE"
+
+        deployment_preference["StateMachineVersionArn"] = state_machine_version_arn
+        state_machine_alias.DeploymentPreference = deployment_preference
+
+        return state_machine_alias
+
+    def _generate_managed_traffic_shifting_resources(
+        self,
+    ) -> List[Any]:
+        """Generates and returns the version and alias resources associated with this state machine's managed traffic shifting.
+
+        :returns: a list containing the state machine's version and alias resources
+        :rtype: list
+        """
+        if not self.auto_publish_alias and not self.deployment_preference:
+            return []
+        if not self.auto_publish_alias and self.deployment_preference:
+            raise InvalidResourceException(
+                self.logical_id, "'DeploymentPreference' requires 'AutoPublishAlias' property to be specified."
+            )
+
+        state_machine_version = self._construct_version()
+        return [state_machine_version, self._construct_alias(state_machine_version)]
+
     def _generate_event_resources(self) -> List[Dict[str, Any]]:
         """Generates and returns the resources associated with this state machine's event sources.
 
@@ -308,9 +384,9 @@ class StateMachineGenerator:
 
         for key, value in sorted(iterator, key=lambda item: item[0]):  # type: ignore[no-any-return]
             if is_intrinsic(value) or is_dynamic_reference(value):
-                dynamic_value_paths.append(path + [key])
+                dynamic_value_paths.append([*path, key])
             elif isinstance(value, (dict, list)):
-                dynamic_value_paths.extend(self._get_paths_to_intrinsics(value, path + [key]))  # type: ignore[no-untyped-call]
+                dynamic_value_paths.extend(self._get_paths_to_intrinsics(value, [*path, key]))  # type: ignore[no-untyped-call]
 
         return dynamic_value_paths
 
