@@ -2,7 +2,7 @@ import copy
 import json
 import logging
 from time import sleep
-from typing import Any, Dict, List, Optional, Tuple
+from typing import Any, Callable, Dict, List, Optional, Tuple
 
 import boto3
 from botocore.client import BaseClient
@@ -28,7 +28,7 @@ PLUGIN_METRICS_PREFIX = "Plugin-ServerlessApp"
 
 class ServerlessAppPlugin(BasePlugin):
     """
-    Resolves all of the ApplicationId and Semantic Version pairs
+    Resolves all the ApplicationId and Semantic Version pairs
     for AWS::Serverless::Application to template URLs.
 
     To retrieve a template from the Serverless Application Repository (SAR),
@@ -57,7 +57,8 @@ class ServerlessAppPlugin(BasePlugin):
         wait_for_template_active_status: bool = False,
         validate_only: bool = False,
         parameters: Optional[Dict[str, Any]] = None,
-    ):
+        sar_client_creator: Optional[Callable[[], BaseClient]] = None,
+    ) -> None:
         """
         Initialize the plugin.
 
@@ -65,6 +66,8 @@ class ServerlessAppPlugin(BasePlugin):
         :param boto3.client sar_client: The boto3 client to use to access the Serverless Application Repository
         :param bool wait_for_template_active_status: Flag to wait for all templates to become active
         :param bool validate_only: Flag to only validate application access (uses get_application API instead)
+        :param bool sar_client_creator: A function to return a SAR client.
+                                        Only used when sar_client is None and SAR calls are made.
         """
         super().__init__()
         if parameters is None:
@@ -72,6 +75,7 @@ class ServerlessAppPlugin(BasePlugin):
         self._applications: Dict[Tuple[str, str], Any] = {}
         self._in_progress_templates: List[Tuple[str, str]] = []
         self.__sar_client = sar_client
+        self._sar_client_creator = sar_client_creator
         self._wait_for_template_active_status = wait_for_template_active_status
         self._validate_only = validate_only
         self._parameters = parameters
@@ -84,11 +88,14 @@ class ServerlessAppPlugin(BasePlugin):
 
     @property
     def _sar_client(self) -> BaseClient:
-        # Lazy initialization of the client- create it when it is needed
+        # Lazy initialization of the client-create it when it is needed
         if not self.__sar_client:
-            # a SAR call could take a while to finish, leaving the read_timeout default (60s).
-            client_config = Config(connect_timeout=BOTO3_CONNECT_TIMEOUT)
-            self.__sar_client = boto3.client("serverlessrepo", config=client_config)
+            if self._sar_client_creator:
+                self.__sar_client = self._sar_client_creator()
+            else:
+                # a SAR call could take a while to finish, leaving the read_timeout default (60s).
+                client_config = Config(connect_timeout=BOTO3_CONNECT_TIMEOUT)
+                self.__sar_client = boto3.client("serverlessrepo", config=client_config)
         return self.__sar_client
 
     @staticmethod
@@ -156,7 +163,7 @@ class ServerlessAppPlugin(BasePlugin):
             except ClientError as e:
                 error_code = e.response["Error"]["Code"]
                 if error_code == "TooManyRequestsException":
-                    LOG.debug("SAR call timed out for application id {}".format(app_id))
+                    LOG.debug(f"SAR call timed out for application id {app_id}")
                     sleep_time = self._get_sleep_time_sec()
                     sleep(sleep_time)
                     self._total_wait_time += sleep_time
@@ -211,14 +218,14 @@ class ServerlessAppPlugin(BasePlugin):
         :param string key: The dictionary key consisting of (ApplicationId, SemanticVersion)
         :param string logical_id: the logical_id of this application resource
         """
-        LOG.info("Getting application {}/{} from serverless application repo...".format(app_id, semver))
+        LOG.info(f"Getting application {app_id}/{semver} from serverless application repo...")
         try:
             self._sar_service_call(self._get_application, logical_id, app_id, semver)
             self._applications[key] = {"Available"}
-            LOG.info("Finished getting application {}/{}.".format(app_id, semver))
+            LOG.info(f"Finished getting application {app_id}/{semver}.")
         except EndpointConnectionError as e:
             # No internet connection. Don't break verification, but do show a warning.
-            warning_message = "{}. Unable to verify access to {}/{}.".format(e, app_id, semver)
+            warning_message = f"{e}. Unable to verify access to {app_id}/{semver}."
             LOG.warning(warning_message)
             self._applications[key] = {"Unable to verify"}
 
@@ -231,10 +238,10 @@ class ServerlessAppPlugin(BasePlugin):
         :param string key: The dictionary key consisting of (ApplicationId, SemanticVersion)
         :param string logical_id: the logical_id of this application resource
         """
-        LOG.info("Requesting to create CFN template {}/{} in serverless application repo...".format(app_id, semver))
+        LOG.info(f"Requesting to create CFN template {app_id}/{semver} in serverless application repo...")
         response = self._sar_service_call(self._create_cfn_template, logical_id, app_id, semver)
 
-        LOG.info("Requested to create CFN template {}/{} in serverless application repo.".format(app_id, semver))
+        LOG.info(f"Requested to create CFN template {app_id}/{semver} in serverless application repo.")
         self._applications[key] = response[self.TEMPLATE_URL_KEY]
         if response["Status"] != "ACTIVE":
             self._in_progress_templates.append((response[self.APPLICATION_ID_KEY], response["TemplateId"]))
@@ -353,7 +360,7 @@ class ServerlessAppPlugin(BasePlugin):
                 except ClientError as e:
                     error_code = e.response["Error"]["Code"]
                     if error_code == "TooManyRequestsException":
-                        LOG.debug("SAR call timed out for application id {}".format(application_id))
+                        LOG.debug(f"SAR call timed out for application id {application_id}")
                         break  # We were throttled by SAR, break out to a sleep
                     raise e
 
